use crate::{DeserializationError, Serializable};

const CR_BUFFER_MIN_CAPACITY: usize = 3;

// Contains the version using semantic versioning
// The patch version is omitted, because it should not affect the communication between the server and the client
// Therefore, a client and a server with a different patch version should be completely compatible
pub struct ClientRegistration {
    pub major_version: u8,
    pub minor_version: u8,
    pub name: String,
}

impl Serializable for ClientRegistration {
    fn serialize(&self, buf: &mut [u8]) {
        let mut vec_buf: Vec<u8> = Vec::with_capacity(buf.len());
        vec_buf.insert(0, self.major_version);
        vec_buf.insert(1, self.minor_version);

        let name_bytes = self.name.as_bytes();
        vec_buf.extend_from_slice(name_bytes);

        buf[..vec_buf.len()].copy_from_slice(&vec_buf);
    }

    fn deserialize(buf: &[u8]) -> Result<Self, DeserializationError> {
        if buf.len() < CR_BUFFER_MIN_CAPACITY {
            return Err(DeserializationError::MissingData);
        }

        Ok(ClientRegistration {
            major_version: buf[0],
            minor_version: buf[1],
            name: String::from_utf8_lossy(&buf[2..]).into_owned(),
        })
    }
}
